/*
tunnel.h

Copyright (C) 1999 Lars Brinkhoff.  See COPYING for terms and conditions.
*/

/*
Tunnel *tunnel_new_client (const char *host, int host_port,
		    const char *proxy, int proxy_port, size_t content_length);

  Create a new HTTP tunnel client.

Tunnel *tunnel_new_server (int port);

  Create a new HTTP tunnel server.

int tunnel_connect (Tunnel *tunnel);

  Open the tunnel.  (Client only.)

int tunnel_accept (Tunnel *tunnel);

  Accept a tunnel connection.  (Server only.)

int tunnel_in_fd (Tunnel *tunnel);
int tunnel_out_fd (Tunnel *tunnel);

  Return the file descriptor used for reading or writing, respectively.
  Warning: don't try to use these with read() or write().  Currently,
  they are only meant to be used with poll() or select().

ssize_t tunnel_read (Tunnel *tunnel, void *data, size_t length);
ssize_t tunnel_write (Tunnel *tunnel, void *data, size_t length);

  Read or write to the tunnel.  Same semantics as with read() and
  write().  Watch out for return values less than length.

int tunnel_padding (Tunnel *tunnel, size_t length);

  Send length pad bytes.

int tunnel_maybe_pad (Tunnel *tunnel, size_t length);

  Pad to nearest even multiple of length.

int tunnel_close (Tunnel *tunnel);

  Close the tunnel.
*/

#ifndef TUNNEL_H
#define TUNNEL_H

#include "config.h"

#include <errno.h>
#include <unistd.h>
#include <netinet/in.h>

#if SIZEOF_CHAR == 1
typedef unsigned char Request;
#else
#error "FIXME: Can't handle SIZEOF_CHAR != 1"
#endif

#if SIZEOF_SHORT == 2
typedef unsigned short Length;
#else
#error "FIXME: Can't handle SIZEOF_SHORT != 2"
#endif

typedef struct tunnel Tunnel;

enum tunnel_request
{
  TUNNEL_OPEN = 0x01,
  TUNNEL_DATA = 0x02,
  TUNNEL_PADDING = 0x03,
  TUNNEL_PAD1 = 0x04,
  TUNNEL_CLOSE = 0x05
};

extern Tunnel *tunnel_new_client (const char *host, int host_port,
				  const char *proxy, int proxy_port,
				  size_t content_length);
extern Tunnel *tunnel_new_server (int port);
extern int tunnel_connect (Tunnel *tunnel);
extern int tunnel_accept (Tunnel *tunnel);
extern void tunnel_read_only (Tunnel *tunnel);
extern void tunnel_write_only (Tunnel *tunnel);
extern int tunnel_in_fd (Tunnel *tunnel);
extern int tunnel_out_fd (Tunnel *tunnel);
extern ssize_t tunnel_read (Tunnel *tunnel, void *data, size_t length);
extern ssize_t tunnel_write (Tunnel *tunnel, void *data, size_t length);
extern int tunnel_padding (Tunnel *tunnel, size_t length);
extern int tunnel_maybe_pad (Tunnel *tunnel, size_t length);
extern int tunnel_close (Tunnel *tunnel);
extern void tunnel_destroy (Tunnel *tunnel);

#endif /* TUNNEL_H */
